-- scope stuff
gearquipper = gearquipper or {};
local c = gearquipper;

GQ_OPTIONS = GQ_OPTIONS or {};
GQ_DATA = GQ_DATA or {};

local paperDollButton, paperDollMenu;
local slotCheckboxes;
local DIALOG_ADDSET = "DIALOG_ADDSET";
local DIALOG_DELETESET = "DIALOG_DELETESET";

-- init
function c:Init()
	c.realmName = GetRealmName();
	c.charName = UnitName("player");
	GQ_DATA[c.realmName] = GQ_DATA[c.realmName] or {};
	GQ_DATA[c.realmName][c.charName] = GQ_DATA[c.realmName][c.charName] or {};
	GQ_DATA[c.realmName][c.charName][c.FIELD_SETS] = GQ_DATA[c.realmName][c.charName][c.FIELD_SETS] or {};

	paperDollButton = c:CreatePaperDollButton();
	paperDollButton:SetScript("OnClick", function() c:TogglePaperDollMenu(); end);

	paperDollMenu = c:CreatePaperDollMenu();

	c:CreateSlotCheckboxes();
	c:CreateDialogs();

	c.initFinished = true;
	c:Println(c:GetText("AddOn loaded."));
end

function c:CreatePaperDollButton()
	local button = CreateFrame("Button", "GQ_PaperDollButton", _G["PaperDollItemsFrame"]);
	button:SetPoint("TOPRIGHT", _G["PaperDollItemsFrame"], "TOPRIGHT", -40, -45);
	button:SetWidth(50);
	button:SetHeight(20);
	button:SetText("GQ");
	button:SetNormalFontObject("GameFontNormal");
	button:RegisterForClicks("LeftButtonUp");

	local ntex = button:CreateTexture();
	ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up");
	ntex:SetTexCoord(0, 0.625, 0, 0.6875);
	ntex:SetAllPoints();
	button:SetNormalTexture(ntex);

	local htex = button:CreateTexture();
	htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight");
	htex:SetTexCoord(0, 0.625, 0, 0.6875);
	htex:SetAllPoints();
	button:SetHighlightTexture(htex);

	local ptex = button:CreateTexture();
	ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down");
	ptex:SetTexCoord(0, 0.625, 0, 0.6875);
	ptex:SetAllPoints();
	button:SetPushedTexture(ptex);

	return button;
end

function c:CreateSlotCheckboxes()
	slotCheckboxes = {};
	for slotId, slotName in pairs(c.slotInfo) do
		slotCheckboxes[slotId] = c:CreateSlotCheckbox(slotName, _G["Character" .. slotName]);
	end
end

function c:CreateSlotCheckbox(slotName, parent)
	local checkbutton = CreateFrame("CheckButton", slotName .. "Checkbox", parent, "UICheckButtonTemplate");
	checkbutton:SetWidth(24);
	checkbutton:SetHeight(24);
	checkbutton:SetPoint("BOTTOMRIGHT", 4, -4);
	checkbutton:RegisterForClicks("LeftButtonUp");
	checkbutton:SetScript("OnClick", function(self) c:SaveSlotActive(c:GetSlotId(slotName), self:GetChecked()) end);
	return checkbutton;
end

local menuStructure;
function c:GetMenuStructure()
	if not menuStructure then
		local switchSet, addSet, removeSet = c:GetText("Switch to set:"), c:GetText("Add set..."), c:GetText("Remove set");
		local setNames = c:LoadSetNames();

		menuStructure = {
			[0] = {
				switchSet,
				"separator1",
				addSet,
				removeSet
			},
			[switchSet] = {},
			["separator1"] = {},
			[addSet] = {},
			[removeSet] = {}
		};

		for _, setName in ipairs(setNames) do
			table.insert(menuStructure[switchSet], { ["name"] = setName;} );
			table.insert(menuStructure[removeSet], { ["name"] = setName;} );
		end
	end
	return menuStructure;
end

function c:CreatePaperDollMenu()
	local dropdown = CreateFrame("Frame", "GQ_PaperDollMenu", UIParent, "UIDropDownMenuTemplate");
	UIDropDownMenu_Initialize(dropdown, function(self, level)
		level = level or 1;

		if (level == 1) then
			for no, key in pairs(c:GetMenuStructure()[0]) do
				local subarray = c:GetMenuStructure()[key];

				local info = UIDropDownMenu_CreateInfo();
				info.hasArrow = table.getn(subarray) > 0;
				info.notCheckable = true;
				info.notClickable = c:StartsWith(key, "separator");
				if not c:StartsWith(key, "separator") then
					info.text = key;
					info.value = {
						["Level1_Key"] = key;
					};
					if key == c:GetText("Add set...") then
						info.func = function()
							StaticPopup_Show(DIALOG_ADDSET);
							c:TogglePaperDollMenu();
						end;
					end
				end
				UIDropDownMenu_AddButton(info, level);
			end
		end

		if (level == 2) then
			local Level1_Key = UIDROPDOWNMENU_MENU_VALUE["Level1_Key"];
			local subarray = c:GetMenuStructure()[Level1_Key];
			local currentSetName = c:LoadCurrentSetName();

			for key, subsubarray in pairs(subarray) do
				local info = UIDropDownMenu_CreateInfo();
				info.hasArrow = false;
				info.text = subsubarray["name"];
				info.value = {
					["Level1_Key"] = Level1_Key;
					["Sublevel_Key"] = key;
				};
				if Level1_Key == c:GetText("Switch to set:") then
					info.notCheckable = false;
					info.checked = currentSetName and currentSetName == subsubarray["name"];
					info.func = function()
						c:SwitchToSet(subsubarray["name"]);
						c:TogglePaperDollMenu();
					end;
				elseif c:GetText("Remove set") then
					info.notCheckable = true;
					info.func = function()
						local dialog = StaticPopup_Show(DIALOG_DELETESET, subsubarray["name"]);
						dialog.data = subsubarray["name"];
						c:TogglePaperDollMenu();
					end;
				end

				UIDropDownMenu_AddButton(info, level);
			end
		end
	end, "MENU");

	return dropdown;
end

function c:CreateDialogs()
	StaticPopupDialogs[DIALOG_ADDSET] = {
		text = c:GetText("Enter a name for this set:"),
		button1 = c:GetText("Apply"),
		button2 = c:GetText("Cancel"),
		hasEditBox = true,
		maxLetters = 30,
		EditBoxOnEnterPressed = function (self)
			local parent = self:GetParent();
			local setName = c:Trim(self:GetText());
			if setName and not tContains(c:LoadSetNames(), setName) then
				c:SaveSet(setName);
				menuStructure = nil;
			end
			parent:Hide();
		end,
		EditBoxOnEscapePressed = function (self)
			self:GetParent():Hide();
		end,
		OnAccept = function(self, data, data2)
			local setName = c:Trim(self.editBox:GetText());
			if setName and not tContains(c:LoadSetNames(), setName) then
				c:SaveSet(setName);
				menuStructure = nil;
			end
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
		};

		StaticPopupDialogs[DIALOG_DELETESET] = {
			text = c:GetText("Do you really want to delete \"%s\"?", "%s"),
			button1 = c:GetText("Apply"),
			button2 = c:GetText("Cancel"),
			EditBoxOnEscapePressed = function (self)
				self:GetParent():Hide();
			end,
			OnAccept = function(self, data)
				if data and tContains(c:LoadSetNames(), data) then
					c:DeleteSet(data);
					menuStructure = nil;
				end
			end,
			timeout = 0,
			whileDead = true,
			hideOnEscape = true,
			preferredIndex = 3
			};
end

-- command handling
SLASH_GEARQUIPPER1 = "/gq";

SlashCmdList["GEARQUIPPER"] = function(msg)

end

-- event handling
c.eventFrame = c.eventFrame or CreateFrame("Frame");
c.eventFrame:RegisterEvent("ADDON_LOADED");
c.eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
c.eventFrame:RegisterEvent("UNIT_INVENTORY_CHANGED");

c.eventFrame:HookScript("OnEvent", function(self, event, msg, ...)
	if event then
		if event == "ADDON_LOADED" and msg == "GearQuipper" and not c.initFinished then
			c:Init();
		elseif event == "PLAYER_ENTERING_WORLD" then
			-- create default gear if not existing
			if table.getn(c:LoadSetNames()) == 0 then
				c:SaveDefaultSet();
				c:SetCheckboxValues(true, true);
				menuStructure = nil;
			else
				c:SetCheckboxValues();
			end
		elseif event == "UNIT_INVENTORY_CHANGED" and msg == "player" then
			local currentSet = c:LoadCurrentSet();
			if currentSet then
				for slotId, slotName in pairs(c.slotInfo) do
					if c:IsSlotActive(slotId) and c:GetItemId(slotId) ~= currentSet[c.FIELD_SLOTS][slotId] then
						c:SaveSlot(slotId);
					end
				end
			end
		end
	end
end);

function c:TogglePaperDollMenu()
	ToggleDropDownMenu(1, nil, paperDollMenu, paperDollButton, -20, 0);
end

function c:SwitchToSet(setName)
	local set = c:LoadSet(setName);
	if set and set[c.FIELD_SLOTS] then
		for slotId, itemId in pairs(set[c.FIELD_SLOTS]) do
			if c:IsSlotActive(slotId) and c:GetItemId(slotId) ~= itemId then
				EquipItemByName(itemId, slotId);
			end
		end
		c:SaveCurrentSetName(setName);
	end
end

function c:SetCheckboxValues(overrideValue, save)
	if overrideValue ~= nil then
		for slotId, checkBox in pairs(slotCheckboxes) do
			c:SetCheckboxValue(slotId, overrideValue, save);
		end
	else
		local activeSlots = c:LoadSlotActive();
		if activeSlots then
			for slotId, value in pairs(activeSlots) do
				c:SetCheckboxValue(slotId, value);
			end
		end
	end
end

function c:SetCheckboxValue(slotId, value, save)
	slotCheckboxes[slotId]:SetChecked(value);
	if save then
		c:SaveSlotActive(slotId, value)
	end
end

function c:IsSlotActive(slotId)
	return slotCheckboxes[slotId]:GetChecked();
end